infobar.warning {
    background-color: #c64600;
}
.webview-filler-title {
    font-weight: 300;
    font-size: 20pt;
    letter-spacing: 0.1rem;
}
.manage-tags-filler-title {
    font-weight: 300;
    font-size: 18pt;
}
.force-background {
    background-color: @theme_bg_color;
}
